from decimal import Decimal

from django.conf import settings
from django.db import models
# Create your models here.
from django.utils.functional import cached_property

from coreapp.base import BaseModel
from inventory.utils import common, image_utils
from inventory.utils.slug_utils import unique_slugify
from coreapp.models import Website


class Slider(BaseModel):
    title = models.CharField(max_length=100)
    image = models.ImageField(upload_to='slider/', blank=True, null=True)
    description = models.TextField(blank=True, null=True)
    url = models.CharField(max_length=100, blank=True, null=True)
    is_active = models.BooleanField(default=True)
    order = models.IntegerField(default=0)

    def __str__(self):
        return self.title


class Category(BaseModel):
    parent_name = models.CharField(max_length=250, blank=True, null=True, editable=False)
    parent = models.ForeignKey('self', on_delete=models.CASCADE, null=True, blank=True)
    name = models.CharField(max_length=100, db_index=True)
    code = models.CharField(max_length=100, db_index=True)
    slug = models.CharField(max_length=100, blank=True, unique=True, db_index=True, editable=True)
    image = models.ImageField(
        upload_to='categories/',
        default='categories/default.png',
        help_text='700x700 (png,jpg,jpeg)'
    )
    is_active = models.BooleanField(default=True)
    product_count = models.IntegerField(default=0, db_index=True)
    is_menu = models.BooleanField(default=False)

    @property
    def get_parent_name(self):
        if self.is_parent:
            return None
        else:
            return common.path_to_root_category(self.parent_id)

    @cached_property
    def is_parent(self):
        return True if self.parent_id is None else False

    def __str__(self):
        return self.name

    @cached_property
    def get_children(self):
        return self.category_set.filter(parent_id=self.id)

    def save(self, *args, **kwargs):
        if self.slug is None or self.slug == '':
            self.slug = unique_slugify(self, self.name)
        if self.parent_id:
            self.parent_name = common.path_to_root_category(self.parent_id)
        super(Category, self).save(**kwargs)


class Brand(BaseModel):
    name = models.CharField(max_length=100, db_index=True)
    code = models.CharField(max_length=100, db_index=True)
    slug = models.CharField(max_length=100, blank=True, unique=True, db_index=True, editable=False)
    image = models.ImageField(
        upload_to='brands/',
        default='brands/default.png',
        help_text='700x700 (png,jpg,jpeg)'
    )
    is_active = models.BooleanField(default=True)
    product_count = models.IntegerField(default=0, db_index=True)

    def __str__(self):
        return self.name

    def save(self, *args, **kwargs):
        if self.slug is None or self.slug == '':
            self.slug = unique_slugify(self, self.name)
        super(Brand, self).save(**kwargs)


class Product(BaseModel):
    thumb_m = models.ImageField(
        upload_to='products/',
        default='products/default.png',
        help_text='510x600 (png,jpg,jpeg)',
        max_length=200
    )
    thumb_b = models.ImageField(
        upload_to='products/',
        default='products/default.png',
        help_text='800x800 (png,jpg,jpeg)',
        max_length=200
    )
    image_1 = models.ImageField(
        upload_to='products/',
        default='products/default.png',
        help_text='800x800 (png,jpg,jpeg)'
    )
    image_2 = models.ImageField(
        upload_to='products/',
        default='products/default.png',
        help_text='800x800 (png,jpg,jpeg)'
    )
    image_3 = models.ImageField(
        upload_to='products/',
        default='products/default.png',
        help_text='800x800 (png,jpg,jpeg)'
    )
    image_4 = models.ImageField(
        upload_to='products/',
        default='products/default.png',
        help_text='800x800 (png,jpg,jpeg)'
    )
    
    name = models.CharField(max_length=100, db_index=True)
    code = models.CharField(max_length=100, db_index=True)
    slug = models.CharField(max_length=100, blank=True, unique=True, db_index=True, editable=True)
    
    quantity = models.IntegerField(default=0)
    cost = models.DecimalField(max_digits=10, decimal_places=2, default=0.00)
    price = models.DecimalField(max_digits=10, decimal_places=2, default=0.00)
    description = models.TextField(blank=True, null=True)
    product_details = models.TextField(blank=True, null=True)
    category = models.ForeignKey(Category, on_delete=models.CASCADE)
    brand = models.ForeignKey(Brand, on_delete=models.CASCADE, null=True, blank=True)
    meta_title = models.CharField(max_length=100, blank=True, null=True)
    meta_description = models.CharField(max_length=250, blank=True, null=True)
    meta_keywords = models.CharField(max_length=250, blank=True, null=True)
    avg_rating = models.DecimalField(max_digits=3, decimal_places=2, default=0.00)
    rating_count = models.IntegerField(default=0)
    promotion_available = models.BooleanField(default=False)
    promotion_quantity = models.IntegerField(default=0)
    promotion_price = models.DecimalField(max_digits=10, decimal_places=2, default=0.00)
    is_active = models.BooleanField(default=True)
    is_featured = models.BooleanField(default=False)
    is_new = models.BooleanField(default=False)
    is_best_seller = models.BooleanField(default=False)
    is_on_sale = models.BooleanField(default=False)

    @property
    def get_category_name(self):
        return self.category.name


    def __str__(self):
        return self.name

    @cached_property
    def get_short_desc(self):
        return self.description[:100]

    @cached_property
    def get_you_may_like(self):
        return Product.objects.filter(
            category=self.category
        ).exclude(id=self.id).order_by('?')[:4]

    @cached_property
    def get_reviews(self):
        return self.rating_set.all().order_by('rating')[:5]
    
    def actual_price(self):
        website = Website.objects.latest('id')
        # Excluding Vat Amount
        vat_p = website.vat / 100 + 1 # vat percentage devided by 100 then add 1
        price = self.price / vat_p # Gross aomount divided by vat
        return price

    def save(self, *args, **kwargs):
        if self.slug is None or self.slug == '':
            self.slug = unique_slugify(self, self.name)
        try:
            thumb_m = self.thumb_m.file
            thumb_m.close()
        except (ValueError, FileNotFoundError):
            if self.id:
                self.thumb_m = image_utils.optimize(self.thumb_m.path, 600, 510, 70, 'products', False)
                self.thumb_b = image_utils.optimize(self.thumb_b.path, 800, 800, 70, 'products', False)
                self.image_1 = image_utils.optimize(self.image_1.path, 800, 800, 70, 'products', False)
                self.image_2 = image_utils.optimize(self.image_2.path, 800, 800, 70, 'products', False)
                self.image_3 = image_utils.optimize(self.image_3.path, 800, 800, 70, 'products', False)
                self.image_4 = image_utils.optimize(self.image_4.path, 800, 800, 70, 'products', False)
        super(Product, self).save(*args, **kwargs)


class Rating(BaseModel):
    product = models.ForeignKey(Product, on_delete=models.CASCADE)
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)
    rating = models.IntegerField(default=0)
    comment = models.TextField(blank=True, null=True)

    def __str__(self):
        return self.product.name

    @property
    def get_user_img(self):
        return self.user.image



    def save(self, *args, **kwargs):
        previous_rating = self.product.rating_count
        previous_avg_rating = self.product.avg_rating
        prev_total = previous_rating * previous_avg_rating
        prev_count = previous_rating + 1
        self.product.avg_rating = (prev_total + Decimal(self.rating)) / (prev_count)
        self.product.rating_count = prev_count
        self.product.save()
        super(Rating, self).save(*args, **kwargs)
